module SemiSmoothNewton

include("mesh.jl")
include("function.jl")
include("operator.jl")
include("image.jl")

function clip_line(r0, r1, l0, l1)
    d_l = -(l1[1] - l0[1])
    d_r = +(l1[1] - l0[1])
    d_b = -(l1[2] - l0[2])
    d_t = +(l1[2] - l0[2])

    t_l = -(r0[1] - l0[1]) / d_l
    t_r = +(r1[1] - l0[1]) / d_r
    t_b = -(r0[2] - l0[2]) / d_b
    t_t = +(r1[2] - l0[2]) / d_t

    t0 = max(
	d_l <= 0 ? t_l : 0.,
	d_r <= 0 ? t_r : 0.,
	d_b <= 0 ? t_b : 0.,
	d_t <= 0 ? t_t : 0.)

    t1 = min(
	d_l >= 0 ? t_l : 1.,
	d_r >= 0 ? t_r : 1.,
	d_b >= 0 ? t_b : 1.,
	d_t >= 0 ? t_t : 1.)

    # tolerance on non-intersection
    t1 < t0 + eps() && return false

    c0 = l0 + t0 * (l1 - l0)
    c1 = l0 + t1 * (l1 - l0)

    return c0, c1
end

"project array data onto a grid function"
function project_array!(u, img::Array)
    vertices = u.mesh.vertices
    mesh_box = (
	reduce((a, b) -> min.(a, b), vertices),
	reduce((a, b) -> max.(a, b), vertices))


    for cell in u.mesh.cells
	cell_box = (
	    reduce((a, b) -> min.(a, b), cell.vertices),
	    reduce((a, b) -> max.(a, b), cell.vertices))

	u[cell]

    end
end



end # module
